import chai from 'chai';
import fg from 'fast-glob';
import { clearContext, defaultSuite } from './suite';
import { context } from './context';
import { afterEachHook, afterFileHook, afterAllHook, afterSuiteHook, beforeEachHook, beforeFileHook, beforeAllHook, beforeSuiteHook } from './hooks';
import { SnapshotPlugin } from './snapshot';
import { DefaultReporter } from './reporters/default';
export async function runTask(task, ctx) {
    var _a, _b;
    const { reporter } = ctx;
    await ((_a = reporter.onTaskBegin) === null || _a === void 0 ? void 0 : _a.call(reporter, task, ctx));
    await beforeEachHook.fire(task);
    if (task.suite.mode === 'skip' || task.mode === 'skip') {
        task.status = 'skip';
    }
    else if (task.suite.mode === 'todo' || task.mode === 'todo') {
        task.status = 'todo';
    }
    else {
        try {
            await task.fn();
            task.status = 'pass';
        }
        catch (e) {
            task.status = 'fail';
            task.error = e;
        }
    }
    await afterEachHook.fire(task);
    await ((_b = reporter.onTaskEnd) === null || _b === void 0 ? void 0 : _b.call(reporter, task, ctx));
}
export async function collectFiles(files) {
    const result = [];
    for (const filepath of files) {
        clearContext();
        await import(filepath);
        const collectors = [defaultSuite, ...context.suites];
        const suites = [];
        const file = {
            filepath,
            suites: [],
        };
        for (const c of collectors) {
            context.currentSuite = c;
            suites.push(await c.collect(file));
        }
        file.suites = suites;
        result.push(file);
    }
    return result;
}
export async function runFile(file, ctx) {
    var _a, _b, _c, _d;
    const { reporter } = ctx;
    await ((_a = reporter.onFileBegin) === null || _a === void 0 ? void 0 : _a.call(reporter, file, ctx));
    await beforeFileHook.fire(file);
    for (const suite of file.suites) {
        await ((_b = reporter.onSuiteBegin) === null || _b === void 0 ? void 0 : _b.call(reporter, suite, ctx));
        await beforeSuiteHook.fire(suite);
        for (const t of suite.tasks)
            await runTask(t, ctx);
        await afterSuiteHook.fire(suite);
        await ((_c = reporter.onSuiteEnd) === null || _c === void 0 ? void 0 : _c.call(reporter, suite, ctx));
    }
    await afterFileHook.fire(file);
    await ((_d = reporter.onFileEnd) === null || _d === void 0 ? void 0 : _d.call(reporter, file, ctx));
}
export async function run(options = {}) {
    var _a, _b, _c;
    const { rootDir = process.cwd() } = options;
    chai.use(await SnapshotPlugin({
        rootDir,
        update: options.updateSnapshot,
    }));
    const paths = await fg(options.includes || ['**/*.{test,spec}.{js,mjs,cjs,ts,mts,cts,jsx,tsx}'], {
        absolute: true,
        cwd: options.rootDir,
        ignore: options.excludes || ['**/node_modules/**', '**/dist/**'],
    });
    if (!paths.length) {
        console.error('No test files found');
        process.exitCode = 1;
        return;
    }
    const reporter = new DefaultReporter();
    await ((_a = reporter.onStart) === null || _a === void 0 ? void 0 : _a.call(reporter, options));
    const files = await collectFiles(paths);
    const ctx = {
        files,
        mode: isOnlyMode(files) ? 'only' : 'all',
        userOptions: options,
        reporter,
    };
    await ((_b = reporter.onCollected) === null || _b === void 0 ? void 0 : _b.call(reporter, ctx));
    await beforeAllHook.fire();
    for (const file of files)
        await runFile(file, ctx);
    await afterAllHook.fire();
    await ((_c = reporter.onFinished) === null || _c === void 0 ? void 0 : _c.call(reporter, ctx));
}
function isOnlyMode(files) {
    return !!files.find(file => file.suites.find(suite => suite.mode === 'only' || suite.tasks.find(t => t.mode === 'only')));
}
